@@ -7,6 +7,7 @@ from qtpy.QtCore import Signal
 
 from .. import core
 from .. import gitcmds
+from .. import gitcfg
 from .. import version
 from ..git import STDOUT
 from . import prefs
@@ -45,6 +46,7 @@ class MainModel(QtCore.QObject):
     mode_none = 'none'  # Default: nothing's happened, do nothing
     mode_worktree = 'worktree'  # Comparing index to worktree
     mode_diffstat = 'diffstat'  # Showing a diffstat
+    mode_display = 'display'  # Displaying arbitrary information
     mode_untracked = 'untracked'  # Dealing with an untracked file
     mode_untracked_diff = 'untracked-diff'  # Diffing an untracked file
     mode_index = 'index'  # Comparing index to last commit
@@ -54,7 +56,7 @@ class MainModel(QtCore.QObject):
     modes_undoable = set((mode_amend, mode_index, mode_worktree))
 
     # Modes where we can partially stage files
-    modes_stageable = set((mode_amend, mode_worktree, mode_untracked_diff))
+    modes_partially_stageable = set((mode_amend, mode_worktree, mode_untracked_diff))
 
     # Modes where we can partially unstage files
     modes_unstageable = set((mode_amend, mode_index))
@@ -103,6 +105,7 @@ class MainModel(QtCore.QObject):
         self.submodules = set()
         self.submodules_list = None  # lazy loaded
 
+        self.error = None  # The last error message.
         self.ref_sort = 0  # (0: version, 1:reverse-chrono)
         self.local_branches = []
         self.remote_branches = []
@@ -120,23 +123,45 @@ class MainModel(QtCore.QObject):
         """Whether we can checkout files from the $head."""
         return self.mode in self.modes_undoable
 
+    def partially_stageable(self):
+        """Whether partial staging should be allowed."""
+        return self.mode in self.modes_partially_stageable
+
     def stageable(self):
         """Whether staging should be allowed."""
-        return self.mode in self.modes_stageable
+        return self.partially_stageable() or self.mode == self.mode_untracked
 
     def all_branches(self):
         return self.local_branches + self.remote_branches
 
     def set_worktree(self, worktree):
+        last_worktree = self.git.paths.worktree
         self.git.set_worktree(worktree)
+
         is_valid = self.git.is_valid()
         if is_valid:
+            reset = last_worktree is None or last_worktree != worktree
             cwd = self.git.getcwd()
             self.project = os.path.basename(cwd)
             self.set_directory(cwd)
             core.chdir(cwd)
-            self.update_config(reset=True)
-            self.worktree_changed.emit()
+            self.update_config(reset=reset)
+
+            # Detect the "git init" scenario by checking for branches.
+            # If no branches exist then we cannot use "git rev-parse" yet.
+            err = None
+            refs = self.git.git_path('refs', 'heads')
+            if core.exists(refs) and core.listdir(refs):
+                # "git rev-parse" exits with a non-zero exit status when the
+                # safe.directory protection is active.
+                status, _, err = self.git.rev_parse('HEAD')
+                is_valid = status == 0
+            if is_valid:
+                self.error = None
+                self.worktree_changed.emit()
+            else:
+                self.error = err
+
         return is_valid
 
     def is_git_lfs_enabled(self):
@@ -299,7 +324,7 @@ class MainModel(QtCore.QObject):
         return not self.local_branches
 
     def _update_remotes(self):
-        self.remotes = self.git.remote()[STDOUT].splitlines()
+        self.remotes = gitcfg.get_remotes(self.cfg)
 
     def _update_branches_and_tags(self):
         context = self.context
