@@ -42,7 +42,7 @@ class DiffSyntaxHighlighter(QtGui.QSyntaxHighlighter):
 
     DIFF_FILE_HEADER_START_RGX = re.compile(r'diff --git a/.* b/.*')
     DIFF_HUNK_HEADER_RGX = re.compile(
-        r'(?:@@ -[0-9,]+ \+[0-9,]+ @@)|' r'(?:@@@ (?:-[0-9,]+ ){2}\+[0-9,]+ @@@)'
+        r'(?:@@ -[0-9,]+ \+[0-9,]+ @@)|(?:@@@ (?:-[0-9,]+ ){2}\+[0-9,]+ @@@)'
     )
     BAD_WHITESPACE_RGX = re.compile(r'\s+$')
 
@@ -485,8 +485,8 @@ class Viewer(QtWidgets.QFrame):
     def render_side_by_side(self):
         # Side-by-side lineup comp
         pixmaps = self.pixmaps
-        width = sum([pixmap.width() for pixmap in pixmaps])
-        height = max([pixmap.height() for pixmap in pixmaps])
+        width = sum(pixmap.width() for pixmap in pixmaps)
+        height = max(pixmap.height() for pixmap in pixmaps)
         image = create_image(width, height)
 
         # Paint each pixmap
@@ -505,8 +505,8 @@ class Viewer(QtWidgets.QFrame):
         if len(pixmaps) == 1:
             return pixmaps[0]
 
-        width = max([pixmap.width() for pixmap in pixmaps])
-        height = max([pixmap.height() for pixmap in pixmaps])
+        width = max(pixmap.width() for pixmap in pixmaps)
+        height = max(pixmap.height() for pixmap in pixmaps)
         image = create_image(width, height)
 
         painter = create_painter(image)
@@ -735,7 +735,7 @@ class DiffEditor(DiffTextEdit):
         enabled = False
         s = self.selection_model.selection()
         model = self.model
-        if model.stageable():
+        if model.partially_stageable():
             item = s.modified[0] if s.modified else None
             if item in model.submodules:
                 pass
@@ -784,7 +784,7 @@ class DiffEditor(DiffTextEdit):
                 self.stage_or_unstage.setText(N_('Unstage'))
             menu.addAction(self.stage_or_unstage)
 
-        if model.stageable():
+        if model.partially_stageable():
             item = s.modified[0] if s.modified else None
             if item in model.submodules:
                 path = core.abspath(item)
@@ -879,7 +879,11 @@ class DiffEditor(DiffTextEdit):
         """setPlainText(str) while retaining scrollbar positions"""
         model = self.model
         mode = model.mode
-        highlight = mode not in (model.mode_none, model.mode_untracked)
+        highlight = mode not in (
+            model.mode_none,
+            model.mode_display,
+            model.mode_untracked,
+        )
         self.highlighter.set_enabled(highlight)
 
         scrollbar = self.verticalScrollBar()
@@ -926,7 +930,7 @@ class DiffEditor(DiffTextEdit):
     def apply_selection(self):
         model = self.model
         s = self.selection_model.single_selection()
-        if model.stageable() and (s.modified or s.untracked):
+        if model.partially_stageable() and (s.modified or s.untracked):
             self.process_diff_selection()
         elif model.unstageable():
             self.process_diff_selection(reverse=True)
@@ -993,7 +997,7 @@ class DiffWidget(QtWidgets.QWidget):
             QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Minimum
         )
 
-        self.gravatar_label = gravatar.GravatarLabel()
+        self.gravatar_label = gravatar.GravatarLabel(self.context, parent=self)
 
         self.author_label = TextLabel()
         self.author_label.setTextFormat(Qt.RichText)
@@ -1056,6 +1060,7 @@ class DiffWidget(QtWidgets.QWidget):
 
     def commits_selected(self, commits):
         if len(commits) != 1:
+            self.clear()
             return
         commit = commits[0]
         oid = self.oid = commit.oid
@@ -1085,6 +1090,14 @@ class DiffWidget(QtWidgets.QWidget):
         self.summary_label.set_text(summary)
         self.gravatar_label.set_email(email)
 
+    def clear(self):
+        self.date_label.set_text('')
+        self.oid_label.set_text('')
+        self.author_label.set_text('')
+        self.summary_label.set_text('')
+        self.gravatar_label.clear()
+        self.diff.clear()
+
     def files_selected(self, filenames):
         if not filenames:
             return
