@@ -1,6 +1,6 @@
 """Text widgets"""
 # pylint: disable=unexpected-keyword-arg
-from __future__ import division, absolute_import, unicode_literals
+from __future__ import absolute_import, division, print_function, unicode_literals
 import math
 
 from qtpy import QtCore
@@ -45,18 +45,22 @@ class LineEdit(QtWidgets.QLineEdit):
         return self._get_value(self)
 
     def set_value(self, value, block=False):
+        """Update the widget to the specified value"""
         if block:
-            blocksig = self.blockSignals(True)
+            with qtutils.BlockSignals(self):
+                self._set_value(value)
+        else:
+            self._set_value(value)
+
+    def _set_value(self, value):
+        """Implementation helper to update the widget to the specified value"""
         pos = self.cursorPosition()
         self.setText(value)
         self.setCursorPosition(pos)
-        if block:
-            self.blockSignals(blocksig)
 
 
 class LineEditCursorPosition(object):
-    """Translate cursorPositionChanged(int,int) into cursorPosition(int,int)
-    """
+    """Translate cursorPositionChanged(int,int) into cursorPosition(int,int)"""
 
     def __init__(self, widget, row):
         self._widget = widget
@@ -89,7 +93,7 @@ class BaseTextEditExtension(QtCore.QObject):
 
     def _init_flags(self):
         widget = self.widget
-        widget.setMinimumSize(QtCore.QSize(1, 1))
+        widget.setMinimumSize(QtCore.QSize(10, 10))
         widget.setWordWrapMode(QtGui.QTextOption.WordWrap)
         widget.setLineWrapMode(widget.NoWrap)
         if self._readonly:
@@ -110,9 +114,15 @@ class BaseTextEditExtension(QtCore.QObject):
         return self._get_value(self.widget)
 
     def set_value(self, value, block=False):
+        """Update the widget to the specified value"""
         if block:
-            blocksig = self.widget.blockSignals(True)
+            with qtutils.BlockSignals(self):
+                self._set_value(value)
+        else:
+            self._set_value(value)
 
+    def _set_value(self, value):
+        """Implementation helper to update the widget to the specified value"""
         # Save cursor position
         offset, selection_text = self.offset_and_selection()
         old_value = get(self.widget)
@@ -145,9 +155,6 @@ class BaseTextEditExtension(QtCore.QObject):
             cursor.movePosition(QtGui.QTextCursor.StartOfLine)
             self.widget.setTextCursor(cursor)
 
-        if block:
-            self.widget.blockSignals(blocksig)
-
     def set_cursor_position(self, new_position):
         cursor = self.widget.textCursor()
         cursor.setPosition(new_position)
@@ -620,10 +627,18 @@ class VimMixin(object):
         self.add_navigation('EndOfLine', hotkeys.END_OF_LINE)
 
         qtutils.add_action(
-            widget, 'PageUp', widget.page_up, hotkeys.SECONDARY_ACTION, hotkeys.UP
+            widget,
+            'PageUp',
+            widget.page_up,
+            hotkeys.SECONDARY_ACTION,
+            hotkeys.TEXT_UP,
         )
         qtutils.add_action(
-            widget, 'PageDown', widget.page_down, hotkeys.PRIMARY_ACTION, hotkeys.DOWN
+            widget,
+            'PageDown',
+            widget.page_down,
+            hotkeys.PRIMARY_ACTION,
+            hotkeys.TEXT_DOWN,
         )
         qtutils.add_action(
             widget,
